Unit IBMAD;
uses dos,crt,graph;

type
  Str80       = string[80];
  IOtype      = (Analog,Binary);

Const
  SelectHighOrderByte              = $0001;  { These constants are the hexa- }
  AnalogReadControlRegisterOffset  = $0000;  { decimal offsets that must be  }
  AnalogReadRegisterOffset         = $2000;  { added to the CardBaseAddress  }
  AnalogStatusRegisterOffset       = $0000;  { to access specific functions  }
  AnalogWriteControlRegisterOffset = $1000;
  AnalogWriteRegisterOffset        = $3000;

  BinaryControlRegisterOffset      = $0000;
  BinaryReadWriteRegisterOffset    = $2000;
  BinaryStatusRegisterOffset       = $0000;

  InterruptControlRegisterOffset   = $D000;
  IssacDeviceNoRegisterOffset      = $C000;

  AnalogDeviceCode                 = $0009;
  BinaryDeviceCode                 = $0008;
  DisableAllInterrupts             = $0000;
  EnableAnalogAD                   = $0001;
  DisableAnalogAD                  = $0000;




Var
  CardNumber            : 0..3;
  InChar                : char;
  CheckInteruptStatus,
  Channel,
  ErrorCode,
  Data,
  X,
  VoltageRange,
  VoltageOffset         : integer;
  Rdata,
  Volts                 : real;
  Quit                  : boolean;
  Instr1,
  Instr2,
  NumberString          : string[30];




Procedure Beep;
begin
  Sound(2200);
  Delay(200);
  NoSound;
end; { Procedure Beep }




Function TestBit                        { Test if bit in target is set (on)   }
  ( var Target ;                        { Target to test                      }
        BitNum : integer                { Bit number to test                  }
                          ) : Boolean;  { Function returns true if bit is set }
var
  Subject : Integer absolute Target;
  Temp    : integer;

begin
Temp := Subject;
if BitNum > 0 then Temp := Temp SHR BitNum;
if Odd(Temp) then TestBit := True
else TestBit := False;
end; { TestBit }




Function ConvertDataToVolts             { Convert raw counts to voltage       }
  (  RawData : real;                    { Raw data to convert to voltage      }
    VoltSpan,                           { Total voltage range                 }
      Offset : integer                  { Negative offset of voltage range    }
                       ) : real;        { Function returns voltage value      }

begin
ConvertDataToVolts := (RawData * VoltSpan / 4096) - Offset;
end;




Function ConvertVoltsToData             { Convert voltage to raw counts       }
  (  Voltage : real;                    { Voltage to convert to raw counts    }
    VoltSpan,                           { Total voltage range                 }
      Offset : integer                  { Negative offset of voltage range    }
                       ) : integer;     { Function returns raw counts         }

begin
ConvertVoltsToData :=  Round ((4096 * (Rdata + Offset)) / VoltSpan);
end;




Function CardBaseAddress                { Convert CardNumber to base address  }
  (CardNumber : integer                 { Card number                         }
                        ) : integer;    { Function returns base address       }

begin
Case CardNumber of
  0 : CardBaseAddress := $02E2;
  1 : CardBaseAddress := $06E2;
  2 : CardBaseAddress := $0AE2;
  3 : CardBaseAddress := $0EE2;
end; { case }
end; { CardBaseAddress }




Procedure DisableBoardInterrupts  { Disable all interrupts. These modules     }
                                  { work by polling and do not use interupts. }
  ( CardNumber : integer );       { Card number to access                     }
var
  BaseAddress,
  CheckInterruptStatus : integer;

begin
BaseAddress := CardBaseAddress(CardNumber);
Port[BaseAddress+InterruptControlRegisterOffset] := DisableAllInterrupts;
Delay(1);
CheckInterruptStatus := Port[BaseAddress+InterruptControlRegisterOffset];
ClrScr;
if (CheckInterruptStatus AND 7) <> 0 then
  begin
  WriteLn('Interrupts NOT cleared.');
  WriteLn('CheckInterruptStatus = ',CheckInterruptStatus);
  WriteLn('Exiting Program');
  Halt;
  end;
end; { DisableBoardInterrupts }




Procedure InitDevice              { Initialize specified data aquisition mode }
  ( CardNumber : integer;         { Card number to access                     }
    Mode       : IOtype           { Mode : ANALOG  - for analog  IO functions }
                         );       {        DIGITAL - for digital IO functions }
var
  BaseAddress : integer;

begin
BaseAddress := CardBaseAddress(CardNumber);
case Mode of
  Analog : Port[BaseAddress+IssacDeviceNoRegisterOffset] := AnalogDeviceCode;
  Binary : Port[BaseAddress+IssacDeviceNoRegisterOffset] := BinaryDeviceCode;
end; { case }
Delay(1);
end; { InitDevice }




Function ReadAnalogData                      { Routine for actual ANALOG read }
  ( CardNumber,                              { Card number to access          }
    Channel     : integer                    { Channel number to access (0-3) }
                          ) : integer;       { Function returns value read    }
var
  BaseAddress,
  LowByteData,
  HighByteData,
  TestStatus      : integer;

begin
BaseAddress := CardBaseAddress(CardNumber);

Port[BaseAddress+AnalogReadControlRegisterOffset] := DisableAnalogAD;

Port [  BaseAddress
      + AnalogReadControlRegisterOffset
      + SelectHighOrderByte             ] := Channel;       { set A/D channel }

Delay(1);                    { *** Wait for channel multiplexer to settle *** }
                             { Delay(1) is equivalent to approximately 1000   }
                             { microseconds on 4.77 MHz CPU. An actual delay  }
                             { of only 120 microseconds is required here for  }
                             { the first time the channel is selected         }

Port [  BaseAddress                                              { enable A/D }
      + AnalogReadControlRegisterOffset ] := EnableAnalogAD;     { conversion }

Port [  BaseAddress
      + AnalogReadControlRegisterOffset
      + SelectHighOrderByte             ] := Channel;       { set A/D channel }

Delay(1);                    { *** Wait for channel multiplexer to settle *** }
                             { An actual delay of only 20 microseconds is     }
                             { required here since the channel has already    }
                             { been selected                                  }


{ **************** Test for completion of A/D conversion ******************** }

repeat
TestStatus := Port [  BaseAddress
                    + AnalogStatusRegisterOffset ];
TestStatus := TestStatus AND 1;               { mask out all but A/D busy bit }
until TestStatus = 0;

{ *************************************************************************** }


Port [  BaseAddress                                            { force A/D    }
      + AnalogReadControlRegisterOffset ] := DisableAnalogAD;  { converter to }
                                                               { read status  }
Port [  BaseAddress
      + AnalogReadControlRegisterOffset
      + SelectHighOrderByte             ] := Channel;   { set channel to read }

LowByteData  := Port [ BaseAddress + AnalogReadRegisterOffset ];       { read }
HighByteData := Port [ BaseAddress + AnalogReadRegisterOffset          { data }
                                   + SelectHighOrderByte      ];

ReadAnalogData := (HighByteData SHL 8) + LowByteData;
end; { ReadAnalogData }




Function ReadBinaryData                     { Routine for actual DIGITAL read }
  (CardNumber : integer)                    { Card number to access           }
                         : integer;         { Function returns the value read }
var
  BaseAddress,
  HiByteValue,
  LoByteValue      : integer;

begin
BaseAddress := CardBaseAddress(CardNumber);
LoByteValue := Port [  BaseAddress
                     + BinaryReadWriteRegisterOffset ];
HiByteValue := Port [  BaseAddress
                     + BinaryReadWriteRegisterOffset
                     + SelectHighOrderByte           ];
ReadBinaryData := ( HiByteValue SHL 8 ) + LoByteValue;
end; { ReadBinaryData }




Procedure WriteAnalogData                   { Routine for actual ANALOG write }
  (CardNumber,                              { Card number to access           }
   Channel,                                 { Channel number to access (0-1)  }
   Value       : integer);                  { Value to write                  }
var
  BaseAddress : integer;

begin
BaseAddress := CardBaseAddress(CardNumber);

Port[  BaseAddress
     + AnalogWriteControlRegisterOffset
     + SelectHighOrderByte              ] := Channel;

Port[  BaseAddress
     + AnalogWriteRegisterOffset        ] := Lo(Value);

Port[  BaseAddress
     + AnalogWriteRegisterOffset
     + SelectHighOrderByte              ] := Hi(Value);
end; { WriteAnalogData }




Procedure WriteBinaryData                  { Routine for actual DIGITAL write }
  (CardNumber,                             { Card number to access            }
   Value       : integer);
var
  BaseAddress : integer;

begin
BaseAddress := CardBaseAddress(CardNumber);

Port[  BaseAddress
     + BinaryReadWriteRegisterOffset ] := Lo(Value);

Port[  BaseAddress
     + BinaryReadWriteRegisterOffset
     + SelectHighOrderByte           ] := Hi(Value);
end; { WriteBinaryData }




Procedure DemoAnalogRead;
begin
Channel       :=  0;
VoltageRange  := 20;
VoltageOffset := 10;

GotoXY(1,3);
Writeln('                    TEST OF ANALOG READ');

InitDevice(CardNumber,Analog);
gotoxy(10,7);
write('Voltage Read = ');

repeat
Quit := False;
GotoXY(25,6);
ClrEol;
GotoXY(10,5);
Write('Enter Analog Channel to Sample (0-3) : ');
ClrEol;

repeat
GotoXY(49,5);
Readln(NumberString);
ClrEol;
Val(NumberString, Channel, ErrorCode);
if (ErrorCode <> 0) or (Channel < 0) or (Channel > 3) then
  begin
  GotoXY(49,5);
  ClrEol;
  Beep;
  end;
until (Channel >= 0) and (Channel <= 3) and (ErrorCode = 0);

GotoXY(10,11);
Write('Hit Any Key to Abort');

repeat
Rdata := ReadAnalogData(CardNumber, Channel);
Rdata := ConvertDataToVolts( Rdata, VoltageRange, VoltageOffset );
GotoXY(40,7);
write(Rdata:5:2);
ClrEol;
Delay(100);
until Keypressed;

GotoXY(10,11);
ClrEol;
GotoXY(10,9);
Write('Continue (Y,N) ? ');

repeat
Inchar := ReadKey;
until Inchar in ['n','N','y','Y'];

if Inchar in ['n','N'] then Quit := True;

GotoXY(18,7);
ClrEol;
GotoXY(10,9);
ClrEol;

until Quit;
end; { DemoAnalogRead }



Procedure DemoAnalogWrite;
begin

Quit    := False;

Channel       := 0;
VoltageRange  := 20;
VoltageOffset := 10;

GotoXY(1,3);
Writeln('                    TEST OF ANALOG WRITE');

InitDevice(CardNumber, Analog);

GotoXY(10,5);
Write('Enter Volts to Output (0.0-10.0) : ');

repeat

repeat
GotoXY(45,5);
Readln(NumberString);
ClrEol;
Val(NumberString, Rdata, ErrorCode);
if (ErrorCode <> 0) or (Rdata < 0) or (Rdata > 10) then
  begin
  GotoXY(45,5);
  ClrEol;
  Beep;
  end;
until (Rdata >= 0) and (Rdata <= 10) and (ErrorCode = 0);

Data := ConvertVoltsToData(Rdata, VoltageRange, VoltageOffset);

WriteAnalogData(CardNumber, Channel, Data);

GotoXY(10,7);
Write('Continue (Y,N) ? ');

repeat
Inchar := ReadKey;
until Inchar in ['n','N','y','Y'];

if Inchar in ['n','N'] then Quit := True;

GotoXY(10,7);
ClrEol;

until Quit;

end; { AnalogWriteDemo }



Procedure DemoDigitalRead;
begin
GotoXY(1,3);
Writeln('                    TEST OF BINARY INPUT');

InitDevice(CardNumber,Binary);

Channel := 0;
for Channel := 0 to 15 do
  begin
  Str(Channel:2,Instr1);
  Instr2 := 'BI ' + Instr1 + ' = ';
  GotoXY(20,Channel+5);
  Write(Instr2);
  end;

repeat
Data := ReadBinaryData(CardNumber);
Channel := 0;
for Channel := 0 to 15 do                           { Test individual bits to }
  begin                                             {  check if channel is in }
  if TestBit(Data, Channel) then Instr1 := 'On '    {  high (on) condition    }
  else Instr1 := 'Off';
  GotoXY(29,Channel+5);
  Write(Instr1);                          { Write out if channel is on or off }
  end;
Delay(200);
until KeyPressed;
end; { DemoDigitalRead }




Procedure DemoDigitalWrite;
begin

CardNumber := 0;
InitDevice(CardNumber,Binary);
WriteBinaryData(CardNumber,7);                        { Write 7 to binary out }

GotoXY(1,3);
Writeln('                    TEST OF BINARY OUTPUT');

GotoXY(1,5);
Writeln('Wrote 7 to binary out. Result should be : BO 0, BO 1, BO 2  ON' );
Writeln('                                          BO 3  thru  BO 15 OFF');
GotoXY(10,8);
Write('Press Any Key to Return to Main Menu');

repeat
until KeyPressed;

end; { DemoDigitalWrite }




begin { main program }
repeat
Clrscr;
GotoXY(15,7);
WriteLn('DEMO OF IBM DATA AQUISITION AND CONTROL BOARD MODES');
GotoXY(20,10);
WriteLn('Select Demo :   1. Analog Read');
GotoXY(20,11);
WriteLn('                2. Analog Write');
GotoXY(20,12);
WriteLn('                3. Digital Read');
GotoXY(20,13);
WriteLn('                4. Digital Write');
GotoXY(20,14);
WriteLn('                Q. Quit         ');

repeat
Inchar := ReadKey;
until Inchar in ['1','2','3','4','q','Q'];
Clrscr;

case Inchar of
  '1' : DemoAnalogRead;
  '2' : DemoAnalogWrite;
  '3' : DemoDigitalRead;
  '4' : DemoDigitalWrite;
end;

until Inchar in ['q','Q'];

end. { main program }
